@@ -3,9 +3,13 @@
 #' @usage Globals(object, ...)
 #'
 #' @param object A named list.
+#'
 #' @param \dots Not used.
 #'
-#' @return An object of class \code{Future}.
+#' @return An object of class \code{Globals}, which is a \emph{named} list
+#' of the value of the globals, where the element names are the names of
+#' the globals.  Attribute \code{where} is a named list of the same length
+#' and with the same names.
 #'
 #' @seealso
 #' The \code{\link{globalsOf}()} function identifies globals
@@ -15,7 +19,7 @@
 #' @export
 Globals <- function(object = list(), ...) {
   if (!is.list(object)) {
-    stop("Argument 'object' is not a list: ", class(object)[1])
+    stopf("Argument 'object' is not a list: %s", class(object)[1])
   }
 
   if (length(object) > 0) {
@@ -27,7 +31,7 @@ Globals <- function(object = list(), ...) {
     }
   }
 
-  where <- attr(object, "where")
+  where <- attr(object, "where", exact = TRUE)
   if (length(object) == 0 && is.null(where)) {
     attr(object, "where") <- where <- list()
   }
@@ -52,13 +56,9 @@ as.Globals.Globals <- function(x, ...) x
 as.Globals.list <- function(x, ...) {
   ## Use the globals environments as the locals?
   ## (with emptyenv() as the fallback)
-  where <- attr(x, "where")
+  where <- attr(x, "where", exact = TRUE)
   if (is.null(where)) {
-    where <- lapply(x, FUN = function(obj) {
-        e <- environment(obj)
-        if (is.null(e)) e <- emptyenv()
-        e
-    })
+    where <- lapply(x, FUN = environment_of)
     names(where) <- names(x)
     attr(x, "where") <- where
   }
@@ -69,7 +69,7 @@ as.Globals.list <- function(x, ...) {
 #' @export
 `names<-.Globals` <- function(x, value) {
   x <- NextMethod()
-  where <- attr(x, "where")
+  where <- attr(x, "where", exact = TRUE)
   names(where) <- names(x)
   attr(x, "where") <- where
   invisible(x)
@@ -77,12 +77,12 @@ as.Globals.list <- function(x, ...) {
 
 #' @export
 `[.Globals` <- function(x, i) {
-  where <- attr(x, "where")
+  where <- attr(x, "where", exact = TRUE)
   res <- NextMethod()
   attr(res, "where") <- where[i]
   class(res) <- class(x)
 
-  where <- attr(res, "where")
+  where <- attr(res, "where", exact = TRUE)
   stop_if_not(
     is.list(where),
     length(where) == length(res),
@@ -95,7 +95,7 @@ as.Globals.list <- function(x, ...) {
 
 #' @export
 `$<-.Globals` <- function(x, name, value) {
-  where <- attr(x, "where")
+  where <- attr(x, "where", exact = TRUE)
 
   ## Remove an element?
   if (is.null(value)) {
@@ -105,16 +105,14 @@ as.Globals.list <- function(x, ...) {
     ## Value must be Globals object of length one
     if (inherits(value, "Globals")) {
       if (length(value) != 1) {
-        stop("Cannot assign Globals object of length different than one: ",
+        stopf("Cannot assign Globals object of length different than one: %s",
              length(value))
       }
       x[[name]] <- value[[1]]
-      where[[name]] <- attr(value, "where")[[1]]
+      where[[name]] <- attr(value, "where", exact = TRUE)[[1]]
     } else {
-      w <- environment(value)
-      if (is.null(w)) w <- emptyenv()
       x[[name]] <- value
-      where[[name]] <- w
+      where[[name]] <- environment_of(value)
     }
   }
 
@@ -128,7 +126,7 @@ as.Globals.list <- function(x, ...) {
 c.Globals <- function(x, ...) {
   args <- list(...)
 
-  where <- attr(x, "where")
+  where <- attr(x, "where", exact = TRUE)
   clazz <- class(x)
   class(x) <- NULL
 
@@ -137,26 +135,21 @@ c.Globals <- function(x, ...) {
     name <- names(args)[kk]
 
     if (inherits(g, "Globals")) {
-      w <- attr(g, "where")
+      w <- attr(g, "where", exact = TRUE)
     } else if (is.list(g)) {
       ## Nothing to do?
       if (length(g) == 0) next
 
       names <- names(g)
       stop_if_not(!is.null(names))
-      w <- lapply(g, FUN = function(obj) {
-        e <- environment(obj)
-        if (is.null(e)) e <- emptyenv()
-        e
-      })
+      w <- lapply(g, FUN = environment_of)
       names(w) <- names
     } else {
       if (is.null(name)) {
-        stop("Can only append named objects to Globals list: ", sQuote(mode(g)))
+        stopf("Can only append named objects to Globals list: %s", sQuote(mode(g)))
       }
       g <- structure(list(g), names = name)
-      e <- environment(g)
-      if (is.null(e)) e <- emptyenv()
+      e <- environment_of(g)
       w <- structure(list(e), names = name)
     }
     where <- c(where, w)
@@ -179,7 +172,7 @@ unique.Globals <- function(x, ...) {
   names <- names(x)
   dups <- duplicated(names)
   if (any(dups)) {
-    where <- attr(x, "where")
+    where <- attr(x, "where", exact = TRUE)
     where <- where[!dups]
     x <- x[!dups]
     attr(x, "where") <- where
